# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO
import satnogsdecoders.process


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Grbalpha(KaitaiStruct):
    """:field callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field ssid_mask: ax25_frame.ax25_header.dest_ssid_raw.ssid_mask
    :field ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field src_callsign_raw_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid_raw_ssid_mask: ax25_frame.ax25_header.src_ssid_raw.ssid_mask
    :field src_ssid_raw_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field repeater_rpt_instance_rpt_callsign_raw_callsign: ax25_frame.ax25_header.repeater.rpt_instance.rpt_callsign_raw.callsign_ror.callsign
    :field repeater_rpt_instance_rpt_ssid_raw_ssid_mask: ax25_frame.ax25_header.repeater.rpt_instance.rpt_ssid_raw.ssid_mask
    :field repeater_rpt_instance_rpt_ssid_raw_ssid: ax25_frame.ax25_header.repeater.rpt_instance.rpt_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field pkt_type: ax25_frame.payload.ax25_info.body.pkt_type
    :field uptime_total: ax25_frame.payload.ax25_info.body.uptime_total
    :field uptime_since_last: ax25_frame.payload.ax25_info.body.uptime_since_last
    :field reset_count: ax25_frame.payload.ax25_info.body.reset_count
    :field mcu_10mv: ax25_frame.payload.ax25_info.body.mcu_10mv
    :field batt: ax25_frame.payload.ax25_info.body.batt
    :field temp_cpu: ax25_frame.payload.ax25_info.body.temp_cpu
    :field temp_pa_ntc: ax25_frame.payload.ax25_info.body.temp_pa_ntc
    :field sig_rx_immediate: ax25_frame.payload.ax25_info.body.sig_rx_immediate
    :field sig_rx_avg: ax25_frame.payload.ax25_info.body.sig_rx_avg
    :field sig_rx_max: ax25_frame.payload.ax25_info.body.sig_rx_max
    :field sig_background_avg: ax25_frame.payload.ax25_info.body.sig_background_avg
    :field sig_background_immediate: ax25_frame.payload.ax25_info.body.sig_background_immediate
    :field sig_background_max: ax25_frame.payload.ax25_info.body.sig_background_max
    :field rf_packets_received: ax25_frame.payload.ax25_info.body.rf_packets_received
    :field rf_packets_transmitted: ax25_frame.payload.ax25_info.body.rf_packets_transmitted
    :field ax25_packets_received: ax25_frame.payload.ax25_info.body.ax25_packets_received
    :field ax25_packets_transmitted: ax25_frame.payload.ax25_info.body.ax25_packets_transmitted
    :field digipeater_rx_count: ax25_frame.payload.ax25_info.body.digipeater_rx_count
    :field digipeater_tx_count: ax25_frame.payload.ax25_info.body.digipeater_tx_count
    :field csp_received: ax25_frame.payload.ax25_info.body.csp_received
    :field csp_transmitted: ax25_frame.payload.ax25_info.body.csp_transmitted
    :field i2c1_received: ax25_frame.payload.ax25_info.body.i2c1_received
    :field i2c1_transmitted: ax25_frame.payload.ax25_info.body.i2c1_transmitted
    :field i2c2_received: ax25_frame.payload.ax25_info.body.i2c2_received
    :field i2c2_transmitted: ax25_frame.payload.ax25_info.body.i2c2_transmitted
    :field rs485_received: ax25_frame.payload.ax25_info.body.rs485_received
    :field rs485_transmitted: ax25_frame.payload.ax25_info.body.rs485_transmitted
    :field csp_mcu_received: ax25_frame.payload.ax25_info.body.csp_mcu_received
    :field csp_mcu_transmitted: ax25_frame.payload.ax25_info.body.csp_mcu_transmitted
    :field obc_timestamp: ax25_frame.payload.ax25_info.body.bytes.obc_timestamp
    :field obc_temp: ax25_frame.payload.ax25_info.body.bytes.obc_temp
    :field obc_tmp112_xp: ax25_frame.payload.ax25_info.body.bytes.obc_tmp112_xp
    :field obc_tmp112_yp: ax25_frame.payload.ax25_info.body.bytes.obc_tmp112_yp
    :field obc_tmp112_xn: ax25_frame.payload.ax25_info.body.bytes.obc_tmp112_xn
    :field obc_tmp112_yn: ax25_frame.payload.ax25_info.body.bytes.obc_tmp112_yn
    :field obc_tmp112_zp: ax25_frame.payload.ax25_info.body.bytes.obc_tmp112_zp
    :field obc_mag_mmc_x: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mmc_x
    :field obc_mag_mmc_y: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mmc_y
    :field obc_mag_mmc_z: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mmc_z
    :field obc_mag_mpu_x: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mpu_x
    :field obc_mag_mpu_y: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mpu_y
    :field obc_mag_mpu_z: ax25_frame.payload.ax25_info.body.bytes.obc_mag_mpu_z
    :field obc_mpu_temp: ax25_frame.payload.ax25_info.body.bytes.obc_mpu_temp
    :field obc_gyr_mpu_x: ax25_frame.payload.ax25_info.body.bytes.obc_gyr_mpu_x
    :field obc_gyr_mpu_y: ax25_frame.payload.ax25_info.body.bytes.obc_gyr_mpu_y
    :field obc_gyr_mpu_z: ax25_frame.payload.ax25_info.body.bytes.obc_gyr_mpu_z
    :field obc_acc_mpu_x: ax25_frame.payload.ax25_info.body.bytes.obc_acc_mpu_x
    :field obc_acc_mpu_y: ax25_frame.payload.ax25_info.body.bytes.obc_acc_mpu_y
    :field obc_acc_mpu_z: ax25_frame.payload.ax25_info.body.bytes.obc_acc_mpu_z
    :field obc_uptime_rst: ax25_frame.payload.ax25_info.body.bytes.obc_uptime_rst
    :field obc_uptime_total: ax25_frame.payload.ax25_info.body.bytes.obc_uptime_total
    :field obc_rst_cnt: ax25_frame.payload.ax25_info.body.bytes.obc_rst_cnt
    :field obc_packet_rec_cnt: ax25_frame.payload.ax25_info.body.bytes.obc_packet_rec_cnt
    :field obc_suns_temp_yn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_temp_yn
    :field obc_suns_temp_yp: ax25_frame.payload.ax25_info.body.bytes.obc_suns_temp_yp
    :field obc_suns_temp_xp: ax25_frame.payload.ax25_info.body.bytes.obc_suns_temp_xp
    :field obc_suns_temp_xn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_temp_xn
    :field obc_suns_temp_zn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_temp_zn
    :field obc_suns_irad_yn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_irad_yn
    :field obc_suns_irad_yp: ax25_frame.payload.ax25_info.body.bytes.obc_suns_irad_yp
    :field obc_suns_irad_xp: ax25_frame.payload.ax25_info.body.bytes.obc_suns_irad_xp
    :field obc_suns_irad_xn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_irad_xn
    :field obc_suns_irad_zn: ax25_frame.payload.ax25_info.body.bytes.obc_suns_irad_zn
    :field gps_rst_cnt: ax25_frame.payload.ax25_info.body.bytes.gps_rst_cnt
    :field gps_fix_quality: ax25_frame.payload.ax25_info.body.bytes.gps_fix_quality
    :field gps_tracked: ax25_frame.payload.ax25_info.body.bytes.gps_tracked
    :field gps_temp: ax25_frame.payload.ax25_info.body.bytes.gps_temp
    :field obc_free_mem: ax25_frame.payload.ax25_info.body.bytes.obc_free_mem
    :field obc_crc: ax25_frame.payload.ax25_info.body.bytes.obc_crc
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Grbalpha.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Grbalpha.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Grbalpha.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Grbalpha.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Grbalpha.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Grbalpha.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Grbalpha.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Grbalpha.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Grbalpha.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Grbalpha.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Grbalpha.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Grbalpha.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Grbalpha.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Grbalpha.Tlm(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Grbalpha.Tlm(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class ObcBytes(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.obc_timestamp = self._io.read_u4le()
            self.obc_temp = self._io.read_s2le()
            self.obc_tmp112_xp = self._io.read_s2le()
            self.obc_tmp112_yp = self._io.read_s2le()
            self.obc_tmp112_xn = self._io.read_s2le()
            self.obc_tmp112_yn = self._io.read_s2le()
            self.obc_tmp112_zp = self._io.read_s2le()
            self.obc_mag_mmc_x = self._io.read_s2le()
            self.obc_mag_mmc_y = self._io.read_s2le()
            self.obc_mag_mmc_z = self._io.read_s2le()
            self.obc_mag_mpu_x = self._io.read_s2le()
            self.obc_mag_mpu_y = self._io.read_s2le()
            self.obc_mag_mpu_z = self._io.read_s2le()
            self.obc_mpu_temp = self._io.read_f4le()
            self.obc_gyr_mpu_x = self._io.read_s2le()
            self.obc_gyr_mpu_y = self._io.read_s2le()
            self.obc_gyr_mpu_z = self._io.read_s2le()
            self.obc_acc_mpu_x = self._io.read_s2le()
            self.obc_acc_mpu_y = self._io.read_s2le()
            self.obc_acc_mpu_z = self._io.read_s2le()
            self.obc_uptime_rst = self._io.read_u4le()
            self.obc_uptime_total = self._io.read_u4le()
            self.obc_rst_cnt = self._io.read_u4le()
            self.obc_packet_rec_cnt = self._io.read_u4le()
            self.obc_suns_temp_yn = self._io.read_u2le()
            self.obc_suns_temp_yp = self._io.read_u2le()
            self.obc_suns_temp_xp = self._io.read_u2le()
            self.obc_suns_temp_xn = self._io.read_u2le()
            self.obc_suns_temp_zn = self._io.read_u2le()
            self.obc_suns_irad_yn = self._io.read_u2le()
            self.obc_suns_irad_yp = self._io.read_u2le()
            self.obc_suns_irad_xp = self._io.read_u2le()
            self.obc_suns_irad_xn = self._io.read_u2le()
            self.obc_suns_irad_zn = self._io.read_u2le()
            self.gps_rst_cnt = self._io.read_u4le()
            self.gps_fix_quality = self._io.read_u1()
            self.gps_tracked = self._io.read_u1()
            self.gps_temp = self._io.read_s2le()
            self.obc_free_mem = self._io.read_u2le()
            self.obc_crc = self._io.read_u2le()


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Grbalpha.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Grbalpha.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Grbalpha.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class Comd(KaitaiStruct):
        """
        .. seealso::
           Source - https://needronix.eu/products/cormorant/hamradio-user-guide/
        """
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.skip_first_comma = self._io.read_u1()
            self.pkt_type = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_uptime = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uptime_total_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.uptime_since_last_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_resets = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.reset_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_mcuv = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.mcu_10mv_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_battv = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.batt_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_temp = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.temp_cpu_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.temp_pa_ntc_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_sig = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_rx_immediate_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_rx_avg_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_rx_max_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_immediate_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_avg_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.sig_background_max_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_rf = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rf_packets_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rf_packets_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_ax25 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.ax25_packets_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.ax25_packets_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_digi = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.digipeater_rx_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.digipeater_tx_count_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_csp = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_i2c1 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c1_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c1_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_i2c2 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c2_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.i2c2_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_rs485 = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rs485_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.rs485_transmitted_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.pass_csp_mcu = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_mcu_received_raw = (self._io.read_bytes_term(44, False, True, True)).decode(u"utf8")
            self.csp_mcu_transmitted_raw = (self._io.read_bytes_full()).decode(u"utf8")

        @property
        def sig_rx_max(self):
            if hasattr(self, '_m_sig_rx_max'):
                return self._m_sig_rx_max if hasattr(self, '_m_sig_rx_max') else None

            self._m_sig_rx_max = int(self.sig_rx_max_raw)
            return self._m_sig_rx_max if hasattr(self, '_m_sig_rx_max') else None

        @property
        def temp_pa_ntc(self):
            if hasattr(self, '_m_temp_pa_ntc'):
                return self._m_temp_pa_ntc if hasattr(self, '_m_temp_pa_ntc') else None

            self._m_temp_pa_ntc = int(self.temp_pa_ntc_raw)
            return self._m_temp_pa_ntc if hasattr(self, '_m_temp_pa_ntc') else None

        @property
        def csp_transmitted(self):
            if hasattr(self, '_m_csp_transmitted'):
                return self._m_csp_transmitted if hasattr(self, '_m_csp_transmitted') else None

            self._m_csp_transmitted = int(self.csp_transmitted_raw)
            return self._m_csp_transmitted if hasattr(self, '_m_csp_transmitted') else None

        @property
        def batt(self):
            if hasattr(self, '_m_batt'):
                return self._m_batt if hasattr(self, '_m_batt') else None

            self._m_batt = int(self.batt_raw)
            return self._m_batt if hasattr(self, '_m_batt') else None

        @property
        def sig_rx_avg(self):
            if hasattr(self, '_m_sig_rx_avg'):
                return self._m_sig_rx_avg if hasattr(self, '_m_sig_rx_avg') else None

            self._m_sig_rx_avg = int(self.sig_rx_avg_raw)
            return self._m_sig_rx_avg if hasattr(self, '_m_sig_rx_avg') else None

        @property
        def sig_background_immediate(self):
            if hasattr(self, '_m_sig_background_immediate'):
                return self._m_sig_background_immediate if hasattr(self, '_m_sig_background_immediate') else None

            self._m_sig_background_immediate = int(self.sig_background_immediate_raw)
            return self._m_sig_background_immediate if hasattr(self, '_m_sig_background_immediate') else None

        @property
        def uptime_total(self):
            if hasattr(self, '_m_uptime_total'):
                return self._m_uptime_total if hasattr(self, '_m_uptime_total') else None

            self._m_uptime_total = int(self.uptime_total_raw)
            return self._m_uptime_total if hasattr(self, '_m_uptime_total') else None

        @property
        def rs485_received(self):
            if hasattr(self, '_m_rs485_received'):
                return self._m_rs485_received if hasattr(self, '_m_rs485_received') else None

            self._m_rs485_received = int(self.rs485_received_raw)
            return self._m_rs485_received if hasattr(self, '_m_rs485_received') else None

        @property
        def i2c1_received(self):
            if hasattr(self, '_m_i2c1_received'):
                return self._m_i2c1_received if hasattr(self, '_m_i2c1_received') else None

            self._m_i2c1_received = int(self.i2c1_received_raw)
            return self._m_i2c1_received if hasattr(self, '_m_i2c1_received') else None

        @property
        def temp_cpu(self):
            if hasattr(self, '_m_temp_cpu'):
                return self._m_temp_cpu if hasattr(self, '_m_temp_cpu') else None

            self._m_temp_cpu = int(self.temp_cpu_raw)
            return self._m_temp_cpu if hasattr(self, '_m_temp_cpu') else None

        @property
        def ax25_packets_transmitted(self):
            if hasattr(self, '_m_ax25_packets_transmitted'):
                return self._m_ax25_packets_transmitted if hasattr(self, '_m_ax25_packets_transmitted') else None

            self._m_ax25_packets_transmitted = int(self.ax25_packets_transmitted_raw)
            return self._m_ax25_packets_transmitted if hasattr(self, '_m_ax25_packets_transmitted') else None

        @property
        def ax25_packets_received(self):
            if hasattr(self, '_m_ax25_packets_received'):
                return self._m_ax25_packets_received if hasattr(self, '_m_ax25_packets_received') else None

            self._m_ax25_packets_received = int(self.ax25_packets_received_raw)
            return self._m_ax25_packets_received if hasattr(self, '_m_ax25_packets_received') else None

        @property
        def digipeater_tx_count(self):
            if hasattr(self, '_m_digipeater_tx_count'):
                return self._m_digipeater_tx_count if hasattr(self, '_m_digipeater_tx_count') else None

            self._m_digipeater_tx_count = int(self.digipeater_tx_count_raw)
            return self._m_digipeater_tx_count if hasattr(self, '_m_digipeater_tx_count') else None

        @property
        def csp_mcu_transmitted(self):
            if hasattr(self, '_m_csp_mcu_transmitted'):
                return self._m_csp_mcu_transmitted if hasattr(self, '_m_csp_mcu_transmitted') else None

            self._m_csp_mcu_transmitted = int(self.csp_mcu_transmitted_raw)
            return self._m_csp_mcu_transmitted if hasattr(self, '_m_csp_mcu_transmitted') else None

        @property
        def csp_mcu_received(self):
            if hasattr(self, '_m_csp_mcu_received'):
                return self._m_csp_mcu_received if hasattr(self, '_m_csp_mcu_received') else None

            self._m_csp_mcu_received = int(self.csp_mcu_received_raw)
            return self._m_csp_mcu_received if hasattr(self, '_m_csp_mcu_received') else None

        @property
        def i2c1_transmitted(self):
            if hasattr(self, '_m_i2c1_transmitted'):
                return self._m_i2c1_transmitted if hasattr(self, '_m_i2c1_transmitted') else None

            self._m_i2c1_transmitted = int(self.i2c1_transmitted_raw)
            return self._m_i2c1_transmitted if hasattr(self, '_m_i2c1_transmitted') else None

        @property
        def mcu_10mv(self):
            if hasattr(self, '_m_mcu_10mv'):
                return self._m_mcu_10mv if hasattr(self, '_m_mcu_10mv') else None

            self._m_mcu_10mv = int(self.mcu_10mv_raw)
            return self._m_mcu_10mv if hasattr(self, '_m_mcu_10mv') else None

        @property
        def uptime_since_last(self):
            if hasattr(self, '_m_uptime_since_last'):
                return self._m_uptime_since_last if hasattr(self, '_m_uptime_since_last') else None

            self._m_uptime_since_last = int(self.uptime_since_last_raw)
            return self._m_uptime_since_last if hasattr(self, '_m_uptime_since_last') else None

        @property
        def sig_background_max(self):
            if hasattr(self, '_m_sig_background_max'):
                return self._m_sig_background_max if hasattr(self, '_m_sig_background_max') else None

            self._m_sig_background_max = int(self.sig_background_max_raw)
            return self._m_sig_background_max if hasattr(self, '_m_sig_background_max') else None

        @property
        def sig_rx_immediate(self):
            if hasattr(self, '_m_sig_rx_immediate'):
                return self._m_sig_rx_immediate if hasattr(self, '_m_sig_rx_immediate') else None

            self._m_sig_rx_immediate = int(self.sig_rx_immediate_raw)
            return self._m_sig_rx_immediate if hasattr(self, '_m_sig_rx_immediate') else None

        @property
        def reset_count(self):
            if hasattr(self, '_m_reset_count'):
                return self._m_reset_count if hasattr(self, '_m_reset_count') else None

            self._m_reset_count = int(self.reset_count_raw)
            return self._m_reset_count if hasattr(self, '_m_reset_count') else None

        @property
        def rs485_transmitted(self):
            if hasattr(self, '_m_rs485_transmitted'):
                return self._m_rs485_transmitted if hasattr(self, '_m_rs485_transmitted') else None

            self._m_rs485_transmitted = int(self.rs485_transmitted_raw)
            return self._m_rs485_transmitted if hasattr(self, '_m_rs485_transmitted') else None

        @property
        def rf_packets_received(self):
            if hasattr(self, '_m_rf_packets_received'):
                return self._m_rf_packets_received if hasattr(self, '_m_rf_packets_received') else None

            self._m_rf_packets_received = int(self.rf_packets_received_raw)
            return self._m_rf_packets_received if hasattr(self, '_m_rf_packets_received') else None

        @property
        def rf_packets_transmitted(self):
            if hasattr(self, '_m_rf_packets_transmitted'):
                return self._m_rf_packets_transmitted if hasattr(self, '_m_rf_packets_transmitted') else None

            self._m_rf_packets_transmitted = int(self.rf_packets_transmitted_raw)
            return self._m_rf_packets_transmitted if hasattr(self, '_m_rf_packets_transmitted') else None

        @property
        def digipeater_rx_count(self):
            if hasattr(self, '_m_digipeater_rx_count'):
                return self._m_digipeater_rx_count if hasattr(self, '_m_digipeater_rx_count') else None

            self._m_digipeater_rx_count = int(self.digipeater_rx_count_raw)
            return self._m_digipeater_rx_count if hasattr(self, '_m_digipeater_rx_count') else None

        @property
        def sig_background_avg(self):
            if hasattr(self, '_m_sig_background_avg'):
                return self._m_sig_background_avg if hasattr(self, '_m_sig_background_avg') else None

            self._m_sig_background_avg = int(self.sig_background_avg_raw)
            return self._m_sig_background_avg if hasattr(self, '_m_sig_background_avg') else None

        @property
        def i2c2_received(self):
            if hasattr(self, '_m_i2c2_received'):
                return self._m_i2c2_received if hasattr(self, '_m_i2c2_received') else None

            self._m_i2c2_received = int(self.i2c2_received_raw)
            return self._m_i2c2_received if hasattr(self, '_m_i2c2_received') else None

        @property
        def i2c2_transmitted(self):
            if hasattr(self, '_m_i2c2_transmitted'):
                return self._m_i2c2_transmitted if hasattr(self, '_m_i2c2_transmitted') else None

            self._m_i2c2_transmitted = int(self.i2c2_transmitted_raw)
            return self._m_i2c2_transmitted if hasattr(self, '_m_i2c2_transmitted') else None

        @property
        def csp_received(self):
            if hasattr(self, '_m_csp_received'):
                return self._m_csp_received if hasattr(self, '_m_csp_received') else None

            self._m_csp_received = int(self.csp_received_raw)
            return self._m_csp_received if hasattr(self, '_m_csp_received') else None


    class Obc(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_bytes = self._io.read_bytes(124)
            _process = satnogsdecoders.process.B64decode()
            self._raw_bytes = _process.decode(self._raw__raw_bytes)
            _io__raw_bytes = KaitaiStream(BytesIO(self._raw_bytes))
            self.bytes = Grbalpha.ObcBytes(_io__raw_bytes, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Grbalpha.Callsign(_io__raw_callsign_ror, self, self._root)


    class Tlm(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            _on = self.packet_type_q
            if _on == 44:
                self.body = Grbalpha.Comd(self._io, self, self._root)
            else:
                self.body = Grbalpha.Obc(self._io, self, self._root)

        @property
        def packet_type_q(self):
            if hasattr(self, '_m_packet_type_q'):
                return self._m_packet_type_q if hasattr(self, '_m_packet_type_q') else None

            _pos = self._io.pos()
            self._io.seek(0)
            self._m_packet_type_q = self._io.read_u1()
            self._io.seek(_pos)
            return self._m_packet_type_q if hasattr(self, '_m_packet_type_q') else None






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Cas4(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field pwr_volt: ax25_frame.payload.ax25_info.telemetry.pwr_volt
    :field pwr_cur: ax25_frame.payload.ax25_info.telemetry.pwr_cur
    :field convert_volt: ax25_frame.payload.ax25_info.telemetry.convert_volt
    :field convert_cur: ax25_frame.payload.ax25_info.telemetry.convert_cur
    :field obc_temp: ax25_frame.payload.ax25_info.telemetry.obc_temp
    :field rf_amp_temp: ax25_frame.payload.ax25_info.telemetry.rf_amp_temp
    :field rcv_agc_volt: ax25_frame.payload.ax25_info.telemetry.rcv_agc_volt
    :field rf_fwd: ax25_frame.payload.ax25_info.telemetry.rf_fwd
    :field rf_ref: ax25_frame.payload.ax25_info.telemetry.rf_ref
    :field obc_volt: ax25_frame.payload.ax25_info.telemetry.obc_volt
    :field obc_reset: ax25_frame.payload.ax25_info.telemetry.obc_reset
    :field pkt_cnt: ax25_frame.payload.ax25_info.telemetry.pkt_cnt
    :field sat_num: ax25_frame.payload.ax25_info.telemetry.sat_num
    :field op_mode: ax25_frame.payload.ax25_info.telemetry.op_mode
    :field pwr_on_op_mode: ax25_frame.payload.ax25_info.telemetry.pwr_on_op_mode
    :field i2c_watchdog: ax25_frame.payload.ax25_info.telemetry.i2c_watchdog
    :field i2c_recon_cnt: ax25_frame.payload.ax25_info.telemetry.i2c_recon_cnt
    :field tc_watchdog: ax25_frame.payload.ax25_info.telemetry.tc_watchdog
    :field tc_reset_cnt: ax25_frame.payload.ax25_info.telemetry.tc_reset_cnt
    :field adc_watchdog: ax25_frame.payload.ax25_info.telemetry.adc_watchdog
    :field adc_reset_cnt: ax25_frame.payload.ax25_info.telemetry.adc_reset_cnt
    :field spi_watchdog: ax25_frame.payload.ax25_info.telemetry.spi_watchdog
    :field spi_init_cnt: ax25_frame.payload.ax25_info.telemetry.spi_init_cnt
    :field cpu_watchdog: ax25_frame.payload.ax25_info.telemetry.cpu_watchdog
    :field cpu_reset_cnt: ax25_frame.payload.ax25_info.telemetry.cpu_reset_cnt
    :field framecounter: ax25_frame.payload.ax25_info.framecounter
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Cas4.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Cas4.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Cas4.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Cas4.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Cas4.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Cas4.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Cas4.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Cas4.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Cas4.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Cas4.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Cas4.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Cas4.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            _on = self.pid
            if _on == 240:
                self._raw_ax25_info = self._io.read_bytes_full()
                _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
                self.ax25_info = Cas4.Cas4Frame(_io__raw_ax25_info, self, self._root)
            else:
                self.ax25_info = self._io.read_bytes_full()


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Cas4Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.syncpacket = self._io.read_bytes(2)
            if not self.syncpacket == b"\xEB\x90":
                raise kaitaistruct.ValidationNotEqualError(b"\xEB\x90", self.syncpacket, self._io, u"/types/cas4_frame/seq/0")
            _on = (self.framecounter % 4)
            if _on == 0:
                self._raw_telemetry = self._io.read_bytes(4)
                _io__raw_telemetry = KaitaiStream(BytesIO(self._raw_telemetry))
                self.telemetry = Cas4.Cas4Frame.Frame1(_io__raw_telemetry, self, self._root)
            elif _on == 1:
                self._raw_telemetry = self._io.read_bytes(4)
                _io__raw_telemetry = KaitaiStream(BytesIO(self._raw_telemetry))
                self.telemetry = Cas4.Cas4Frame.Frame2(_io__raw_telemetry, self, self._root)
            elif _on == 3:
                self._raw_telemetry = self._io.read_bytes(4)
                _io__raw_telemetry = KaitaiStream(BytesIO(self._raw_telemetry))
                self.telemetry = Cas4.Cas4Frame.Frame4(_io__raw_telemetry, self, self._root)
            elif _on == 2:
                self._raw_telemetry = self._io.read_bytes(4)
                _io__raw_telemetry = KaitaiStream(BytesIO(self._raw_telemetry))
                self.telemetry = Cas4.Cas4Frame.Frame3(_io__raw_telemetry, self, self._root)
            else:
                self.telemetry = self._io.read_bytes(4)
            self.reserved1 = self._io.read_bytes(9)
            self.framecounterplaceholder = self._io.read_u1()
            self.reserved2 = self._io.read_bytes(112)

        class Frame1(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.pwr_volt = self._io.read_u1()
                self.pwr_cur = self._io.read_u1()
                self.convert_volt = self._io.read_u1()
                self.convert_cur = self._io.read_u1()


        class Frame2(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.obc_temp = self._io.read_u1()
                self.rf_amp_temp = self._io.read_u1()
                self.rcv_agc_volt = self._io.read_u1()
                self.rf_fwd = self._io.read_u1()


        class Frame3(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.rf_ref = self._io.read_u1()
                self.obc_volt = self._io.read_u1()
                self.obc_reset = self._io.read_u1()
                self.pkt_cnt = self._io.read_bits_int_be(4)
                self.sat_num = self._io.read_bits_int_be(4)


        class Frame4(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._read()

            def _read(self):
                self.op_mode = self._io.read_bits_int_be(4)
                self.pwr_on_op_mode = self._io.read_bits_int_be(4)
                self.i2c_watchdog = self._io.read_bits_int_be(1) != 0
                self.i2c_recon_cnt = self._io.read_bits_int_be(3)
                self.tc_watchdog = self._io.read_bits_int_be(1) != 0
                self.tc_reset_cnt = self._io.read_bits_int_be(3)
                self.adc_watchdog = self._io.read_bits_int_be(1) != 0
                self.adc_reset_cnt = self._io.read_bits_int_be(3)
                self.spi_watchdog = self._io.read_bits_int_be(1) != 0
                self.spi_init_cnt = self._io.read_bits_int_be(3)
                self.cpu_watchdog = self._io.read_bits_int_be(1) != 0
                self.cpu_reset_cnt = self._io.read_bits_int_be(3)


        @property
        def framecounter(self):
            if hasattr(self, '_m_framecounter'):
                return self._m_framecounter if hasattr(self, '_m_framecounter') else None

            _pos = self._io.pos()
            self._io.seek(15)
            self._m_framecounter = self._io.read_u1()
            self._io.seek(_pos)
            return self._m_framecounter if hasattr(self, '_m_framecounter') else None


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Cas4.Callsign(_io__raw_callsign_ror, self, self._root)






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Oresat0(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field rpt_callsign: ax25_frame.ax25_header.repeater.rpt_instance[0].rpt_callsign_raw.callsign_ror.callsign
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field aprs_packet_data_type_identifier: ax25_frame.payload.ax25_info.aprs_packet_data_type_identifier
    :field aprs_packet_revision: ax25_frame.payload.ax25_info.aprs_packet_revision
    :field aprs_packet_satellite_id: ax25_frame.payload.ax25_info.aprs_packet_satellite_id
    :field c3_m4_oresat0_state: ax25_frame.payload.ax25_info.c3_m4_oresat0_state
    :field c3_m4_uptime: ax25_frame.payload.ax25_info.c3_m4_uptime
    :field c3_rtc_time: ax25_frame.payload.ax25_info.c3_rtc_time
    :field c3_wdt_num_power_cycles: ax25_frame.payload.ax25_info.c3_wdt_num_power_cycles
    :field c3_emmc_percent_full: ax25_frame.payload.ax25_info.c3_emmc_percent_full
    :field c3_l_rx_bytes_received: ax25_frame.payload.ax25_info.c3_l_rx_bytes_received
    :field c3_l_rx_valid_packets: ax25_frame.payload.ax25_info.c3_l_rx_valid_packets
    :field c3_l_rx_rssi: ax25_frame.payload.ax25_info.c3_l_rx_rssi
    :field c3_uhf_rx_bytes_received: ax25_frame.payload.ax25_info.c3_uhf_rx_bytes_received
    :field c3_uhf_rx_valid_packets: ax25_frame.payload.ax25_info.c3_uhf_rx_valid_packets
    :field c3_uhf_rx_rssi: ax25_frame.payload.ax25_info.c3_uhf_rx_rssi
    :field c3_fw_bank_current_and_next_bank: ax25_frame.payload.ax25_info.c3_fw_bank_current_and_next_bank
    :field c3_l_rx_sequence_number: ax25_frame.payload.ax25_info.c3_l_rx_sequence_number
    :field c3_l_rx_rejected_packets: ax25_frame.payload.ax25_info.c3_l_rx_rejected_packets
    :field battery_pack_1_vbatt: ax25_frame.payload.ax25_info.battery_pack_1_vbatt
    :field battery_pack_1_vcell: ax25_frame.payload.ax25_info.battery_pack_1_vcell
    :field battery_pack_1_vcell_max: ax25_frame.payload.ax25_info.battery_pack_1_vcell_max
    :field battery_pack_1_vcell_min: ax25_frame.payload.ax25_info.battery_pack_1_vcell_min
    :field battery_pack_1_vcell_1: ax25_frame.payload.ax25_info.battery_pack_1_vcell_1
    :field battery_pack_1_vcell_2: ax25_frame.payload.ax25_info.battery_pack_1_vcell_2
    :field battery_pack_1_vcell_avg: ax25_frame.payload.ax25_info.battery_pack_1_vcell_avg
    :field battery_pack_1_temperature: ax25_frame.payload.ax25_info.battery_pack_1_temperature
    :field battery_pack_1_temperature_avg: ax25_frame.payload.ax25_info.battery_pack_1_temperature_avg
    :field battery_pack_1_temperature_max: ax25_frame.payload.ax25_info.battery_pack_1_temperature_max
    :field battery_pack_1_temperature_min: ax25_frame.payload.ax25_info.battery_pack_1_temperature_min
    :field battery_pack_1_current: ax25_frame.payload.ax25_info.battery_pack_1_current
    :field battery_pack_1_current_avg: ax25_frame.payload.ax25_info.battery_pack_1_current_avg
    :field battery_pack_1_current_max: ax25_frame.payload.ax25_info.battery_pack_1_current_max
    :field battery_pack_1_current_min: ax25_frame.payload.ax25_info.battery_pack_1_current_min
    :field battery_pack_1_state: ax25_frame.payload.ax25_info.battery_pack_1_state
    :field battery_pack_1_reported_state_of_charge: ax25_frame.payload.ax25_info.battery_pack_1_reported_state_of_charge
    :field battery_pack_1_full_capacity: ax25_frame.payload.ax25_info.battery_pack_1_full_capacity
    :field battery_pack_1_reported_capacity: ax25_frame.payload.ax25_info.battery_pack_1_reported_capacity
    :field battery_pack_2_vbatt: ax25_frame.payload.ax25_info.battery_pack_2_vbatt
    :field battery_pack_2_vcell: ax25_frame.payload.ax25_info.battery_pack_2_vcell
    :field battery_pack_2_vcell_max: ax25_frame.payload.ax25_info.battery_pack_2_vcell_max
    :field battery_pack_2_vcell_min: ax25_frame.payload.ax25_info.battery_pack_2_vcell_min
    :field battery_pack_2_vcell_1: ax25_frame.payload.ax25_info.battery_pack_2_vcell_1
    :field battery_pack_2_vcell_2: ax25_frame.payload.ax25_info.battery_pack_2_vcell_2
    :field battery_pack_2_vcell_avg: ax25_frame.payload.ax25_info.battery_pack_2_vcell_avg
    :field battery_pack_2_temperature: ax25_frame.payload.ax25_info.battery_pack_2_temperature
    :field battery_pack_2_temperature_avg: ax25_frame.payload.ax25_info.battery_pack_2_temperature_avg
    :field battery_pack_2_temperature_max: ax25_frame.payload.ax25_info.battery_pack_2_temperature_max
    :field battery_pack_2_temperature_min: ax25_frame.payload.ax25_info.battery_pack_2_temperature_min
    :field battery_pack_2_current: ax25_frame.payload.ax25_info.battery_pack_2_current
    :field battery_pack_2_current_avg: ax25_frame.payload.ax25_info.battery_pack_2_current_avg
    :field battery_pack_2_current_max: ax25_frame.payload.ax25_info.battery_pack_2_current_max
    :field battery_pack_2_current_min: ax25_frame.payload.ax25_info.battery_pack_2_current_min
    :field battery_pack_2_state: ax25_frame.payload.ax25_info.battery_pack_2_state
    :field battery_pack_2_reported_state_of_charge: ax25_frame.payload.ax25_info.battery_pack_2_reported_state_of_charge
    :field battery_pack_2_full_capacity: ax25_frame.payload.ax25_info.battery_pack_2_full_capacity
    :field battery_pack_2_reported_capacity: ax25_frame.payload.ax25_info.battery_pack_2_reported_capacity
    :field solar_minus_x_voltage_avg: ax25_frame.payload.ax25_info.solar_minus_x_voltage_avg
    :field solar_minus_x_current_avg: ax25_frame.payload.ax25_info.solar_minus_x_current_avg
    :field solar_minus_x_power_avg: ax25_frame.payload.ax25_info.solar_minus_x_power_avg
    :field solar_minus_x_voltage_max: ax25_frame.payload.ax25_info.solar_minus_x_voltage_max
    :field solar_minus_x_current_max: ax25_frame.payload.ax25_info.solar_minus_x_current_max
    :field solar_minus_x_power_max: ax25_frame.payload.ax25_info.solar_minus_x_power_max
    :field solar_minus_x_energy: ax25_frame.payload.ax25_info.solar_minus_x_energy
    :field solar_minus_y_voltage_avg: ax25_frame.payload.ax25_info.solar_minus_y_voltage_avg
    :field solar_minus_y_current_avg: ax25_frame.payload.ax25_info.solar_minus_y_current_avg
    :field solar_minus_y_power_avg: ax25_frame.payload.ax25_info.solar_minus_y_power_avg
    :field solar_minus_y_voltage_max: ax25_frame.payload.ax25_info.solar_minus_y_voltage_max
    :field solar_minus_y_current_max: ax25_frame.payload.ax25_info.solar_minus_y_current_max
    :field solar_minus_y_power_max: ax25_frame.payload.ax25_info.solar_minus_y_power_max
    :field solar_minus_y_energy: ax25_frame.payload.ax25_info.solar_minus_y_energy
    :field solar_plus_x_voltage_avg: ax25_frame.payload.ax25_info.solar_plus_x_voltage_avg
    :field solar_plus_x_current_avg: ax25_frame.payload.ax25_info.solar_plus_x_current_avg
    :field solar_plus_x_power_avg: ax25_frame.payload.ax25_info.solar_plus_x_power_avg
    :field solar_plus_x_voltage_max: ax25_frame.payload.ax25_info.solar_plus_x_voltage_max
    :field solar_plus_x_current_max: ax25_frame.payload.ax25_info.solar_plus_x_current_max
    :field solar_plus_x_power_max: ax25_frame.payload.ax25_info.solar_plus_x_power_max
    :field solar_plus_x_energy: ax25_frame.payload.ax25_info.solar_plus_x_energy
    :field solar_plus_y_voltage_avg: ax25_frame.payload.ax25_info.solar_plus_y_voltage_avg
    :field solar_plus_y_current_avg: ax25_frame.payload.ax25_info.solar_plus_y_current_avg
    :field solar_plus_y_power_avg: ax25_frame.payload.ax25_info.solar_plus_y_power_avg
    :field solar_plus_y_voltage_max: ax25_frame.payload.ax25_info.solar_plus_y_voltage_max
    :field solar_plus_y_current_max: ax25_frame.payload.ax25_info.solar_plus_y_current_max
    :field solar_plus_y_power_max: ax25_frame.payload.ax25_info.solar_plus_y_power_max
    :field solar_plus_y_energy: ax25_frame.payload.ax25_info.solar_plus_y_energy
    :field star_tracker_emmc_capacity: ax25_frame.payload.ax25_info.star_tracker_emmc_capacity
    :field star_tracker_readable_files: ax25_frame.payload.ax25_info.star_tracker_readable_files
    :field star_tracker_updater_status: ax25_frame.payload.ax25_info.star_tracker_updater_status
    :field star_tracker_updates_cached: ax25_frame.payload.ax25_info.star_tracker_updates_cached
    :field star_tracker_right_ascension: ax25_frame.payload.ax25_info.star_tracker_right_ascension
    :field star_tracker_declination: ax25_frame.payload.ax25_info.star_tracker_declination
    :field star_tracker_roll: ax25_frame.payload.ax25_info.star_tracker_roll
    :field star_tracker_timestamp_of_last_measurement: ax25_frame.payload.ax25_info.star_tracker_timestamp_of_last_measurement
    :field gps_emmc_capacity: ax25_frame.payload.ax25_info.gps_emmc_capacity
    :field gps_readable_files: ax25_frame.payload.ax25_info.gps_readable_files
    :field gps_updater_status: ax25_frame.payload.ax25_info.gps_updater_status
    :field gps_updates_cached: ax25_frame.payload.ax25_info.gps_updates_cached
    :field gps_gps_status: ax25_frame.payload.ax25_info.gps_gps_status
    :field gps_num_of_sats_locked: ax25_frame.payload.ax25_info.gps_num_of_sats_locked
    :field gps_x_position: ax25_frame.payload.ax25_info.gps_x_position
    :field gps_y_postition: ax25_frame.payload.ax25_info.gps_y_postition
    :field gps_z_position: ax25_frame.payload.ax25_info.gps_z_position
    :field gps_x_velocity: ax25_frame.payload.ax25_info.gps_x_velocity
    :field gps_y_velocity: ax25_frame.payload.ax25_info.gps_y_velocity
    :field gps_z_velocity: ax25_frame.payload.ax25_info.gps_z_velocity
    :field gps_timestamp_of_last_packet: ax25_frame.payload.ax25_info.gps_timestamp_of_last_packet
    :field ads_gyro_roll_dot: ax25_frame.payload.ax25_info.ads_gyro_roll_dot
    :field ads_gyro_pitch_dot: ax25_frame.payload.ax25_info.ads_gyro_pitch_dot
    :field ads_gyro_yaw_dot: ax25_frame.payload.ax25_info.ads_gyro_yaw_dot
    :field ads_gyro_imu_temp: ax25_frame.payload.ax25_info.ads_gyro_imu_temp
    :field dxwifi_emmc_capacity: ax25_frame.payload.ax25_info.dxwifi_emmc_capacity
    :field dxwifi_readable_files: ax25_frame.payload.ax25_info.dxwifi_readable_files
    :field dxwifi_updater_status: ax25_frame.payload.ax25_info.dxwifi_updater_status
    :field dxwifi_updates_cached: ax25_frame.payload.ax25_info.dxwifi_updates_cached
    :field dxwifi_transmitting: ax25_frame.payload.ax25_info.dxwifi_transmitting
    :field aprs_packet_crc_minus_32: ax25_frame.payload.ax25_info.aprs_packet_crc_minus_32
    
    Attention: `rpt_callsign` cannot be accessed because `rpt_instance` is an
    array of unknown size at the beginning of the parsing process! Left an
    example in here.
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Oresat0.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Oresat0.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Oresat0.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Oresat0.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Oresat0.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Oresat0.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Oresat0.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Oresat0.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Oresat0.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Oresat0.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Oresat0.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Oresat0.SsidMask(self._io, self, self._root)
            if (self.src_ssid_raw.ssid_mask & 1) == 0:
                self.repeater = Oresat0.Repeater(self._io, self, self._root)

            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Oresat0.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"KJ7SAT") or (self.callsign == u"SPACE ")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self._raw_ax25_info = self._io.read_bytes_full()
            _io__raw_ax25_info = KaitaiStream(BytesIO(self._raw_ax25_info))
            self.ax25_info = Oresat0.Ax25InfoData(_io__raw_ax25_info, self, self._root)


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class Repeaters(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_callsign_raw = Oresat0.CallsignRaw(self._io, self, self._root)
            self.rpt_ssid_raw = Oresat0.SsidMask(self._io, self, self._root)


    class Repeater(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.rpt_instance = []
            i = 0
            while True:
                _ = Oresat0.Repeaters(self._io, self, self._root)
                self.rpt_instance.append(_)
                if (_.rpt_ssid_raw.ssid_mask & 1) == 1:
                    break
                i += 1


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Oresat0.Callsign(_io__raw_callsign_ror, self, self._root)


    class Ax25InfoData(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.aprs_packet_format = (self._io.read_bytes(3)).decode(u"ASCII")
            self.aprs_packet_satellite_id = self._io.read_u1()
            self.aprs_packet_revision = self._io.read_u1()
            self.c3_m4_oresat0_state = (self._io.read_bytes(1)).decode(u"ASCII")
            self.c3_m4_uptime = self._io.read_u4le()
            self.c3_rtc_time = self._io.read_u4le()
            self.c3_wdt_num_power_cycles = self._io.read_u2le()
            self.c3_emmc_percent_full = self._io.read_u1()
            self.c3_l_rx_bytes_received = self._io.read_u4le()
            self.c3_l_rx_valid_packets = self._io.read_u4le()
            self.c3_l_rx_rssi = self._io.read_s1()
            self.c3_uhf_rx_bytes_received = self._io.read_u4le()
            self.c3_uhf_rx_valid_packets = self._io.read_u4le()
            self.c3_uhf_rx_rssi = self._io.read_s1()
            self.c3_fw_bank_current_and_next_bank = self._io.read_u1()
            self.c3_l_rx_sequence_number = self._io.read_u4le()
            self.c3_l_rx_rejected_packets = self._io.read_u4le()
            self.battery_pack_1_vbatt = self._io.read_u2le()
            self.battery_pack_1_vcell = self._io.read_u2le()
            self.battery_pack_1_vcell_max = self._io.read_u2le()
            self.battery_pack_1_vcell_min = self._io.read_u2le()
            self.battery_pack_1_vcell_1 = self._io.read_u2le()
            self.battery_pack_1_vcell_2 = self._io.read_u2le()
            self.battery_pack_1_vcell_avg = self._io.read_u2le()
            self.battery_pack_1_temperature = self._io.read_s2le()
            self.battery_pack_1_temperature_avg = self._io.read_s2le()
            self.battery_pack_1_temperature_max = self._io.read_s2le()
            self.battery_pack_1_temperature_min = self._io.read_s2le()
            self.battery_pack_1_current = self._io.read_s2le()
            self.battery_pack_1_current_avg = self._io.read_s2le()
            self.battery_pack_1_current_max = self._io.read_s2le()
            self.battery_pack_1_current_min = self._io.read_s2le()
            self.battery_pack_1_state = self._io.read_u1()
            self.battery_pack_1_reported_state_of_charge = self._io.read_u1()
            self.battery_pack_1_full_capacity = self._io.read_u2le()
            self.battery_pack_1_reported_capacity = self._io.read_u2le()
            self.battery_pack_2_vbatt = self._io.read_u2le()
            self.battery_pack_2_vcell = self._io.read_u2le()
            self.battery_pack_2_vcell_max = self._io.read_u2le()
            self.battery_pack_2_vcell_min = self._io.read_u2le()
            self.battery_pack_2_vcell_1 = self._io.read_u2le()
            self.battery_pack_2_vcell_2 = self._io.read_u2le()
            self.battery_pack_2_vcell_avg = self._io.read_u2le()
            self.battery_pack_2_temperature = self._io.read_s2le()
            self.battery_pack_2_temperature_avg = self._io.read_s2le()
            self.battery_pack_2_temperature_max = self._io.read_s2le()
            self.battery_pack_2_temperature_min = self._io.read_s2le()
            self.battery_pack_2_current = self._io.read_s2le()
            self.battery_pack_2_current_avg = self._io.read_s2le()
            self.battery_pack_2_current_max = self._io.read_s2le()
            self.battery_pack_2_current_min = self._io.read_s2le()
            self.battery_pack_2_state = self._io.read_u1()
            self.battery_pack_2_reported_state_of_charge = self._io.read_u1()
            self.battery_pack_2_full_capacity = self._io.read_u2le()
            self.battery_pack_2_reported_capacity = self._io.read_u2le()
            self.solar_minus_x_voltage_avg = self._io.read_u2le()
            self.solar_minus_x_current_avg = self._io.read_s2le()
            self.solar_minus_x_power_avg = self._io.read_u2le()
            self.solar_minus_x_voltage_max = self._io.read_u2le()
            self.solar_minus_x_current_max = self._io.read_s2le()
            self.solar_minus_x_power_max = self._io.read_u2le()
            self.solar_minus_x_energy = self._io.read_u2le()
            self.solar_minus_y_voltage_avg = self._io.read_u2le()
            self.solar_minus_y_current_avg = self._io.read_s2le()
            self.solar_minus_y_power_avg = self._io.read_u2le()
            self.solar_minus_y_voltage_max = self._io.read_u2le()
            self.solar_minus_y_current_max = self._io.read_s2le()
            self.solar_minus_y_power_max = self._io.read_u2le()
            self.solar_minus_y_energy = self._io.read_u2le()
            self.solar_plus_x_voltage_avg = self._io.read_u2le()
            self.solar_plus_x_current_avg = self._io.read_s2le()
            self.solar_plus_x_power_avg = self._io.read_u2le()
            self.solar_plus_x_voltage_max = self._io.read_u2le()
            self.solar_plus_x_current_max = self._io.read_s2le()
            self.solar_plus_x_power_max = self._io.read_u2le()
            self.solar_plus_x_energy = self._io.read_u2le()
            self.solar_plus_y_voltage_avg = self._io.read_u2le()
            self.solar_plus_y_current_avg = self._io.read_s2le()
            self.solar_plus_y_power_avg = self._io.read_u2le()
            self.solar_plus_y_voltage_max = self._io.read_u2le()
            self.solar_plus_y_current_max = self._io.read_s2le()
            self.solar_plus_y_power_max = self._io.read_u2le()
            self.solar_plus_y_energy = self._io.read_u2le()
            self.star_tracker_emmc_capacity = self._io.read_u1()
            self.star_tracker_readable_files = self._io.read_u1()
            self.star_tracker_updater_status = self._io.read_u1()
            self.star_tracker_updates_cached = self._io.read_u1()
            self.star_tracker_right_ascension = self._io.read_s2le()
            self.star_tracker_declination = self._io.read_s2le()
            self.star_tracker_roll = self._io.read_s2le()
            self.star_tracker_timestamp_of_last_measurement = self._io.read_u4le()
            self.gps_emmc_capacity = self._io.read_u1()
            self.gps_readable_files = self._io.read_u1()
            self.gps_updater_status = self._io.read_u1()
            self.gps_updates_cached = self._io.read_u1()
            self.gps_gps_status = self._io.read_u1()
            self.gps_num_of_sats_locked = self._io.read_u1()
            self.gps_x_position = self._io.read_s4le()
            self.gps_y_postition = self._io.read_s4le()
            self.gps_z_position = self._io.read_s4le()
            self.gps_x_velocity = self._io.read_s4le()
            self.gps_y_velocity = self._io.read_s4le()
            self.gps_z_velocity = self._io.read_s4le()
            self.gps_timestamp_of_last_packet = self._io.read_u4le()
            self.ads_gyro_roll_dot = self._io.read_s2le()
            self.ads_gyro_pitch_dot = self._io.read_s2le()
            self.ads_gyro_yaw_dot = self._io.read_s2le()
            self.ads_gyro_imu_temp = self._io.read_s1()
            self.dxwifi_emmc_capacity = self._io.read_u1()
            self.dxwifi_readable_files = self._io.read_u1()
            self.dxwifi_updater_status = self._io.read_u1()
            self.dxwifi_updates_cached = self._io.read_u1()
            self.dxwifi_transmitting = self._io.read_bits_int_be(1) != 0
            self._io.align_to_byte()
            self.aprs_packet_crc_minus_32 = self._io.read_u4le()






# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO


if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class Mirsat1(KaitaiStruct):
    """:field dest_callsign: ax25_frame.ax25_header.dest_callsign_raw.callsign_ror.callsign
    :field src_callsign: ax25_frame.ax25_header.src_callsign_raw.callsign_ror.callsign
    :field src_ssid: ax25_frame.ax25_header.src_ssid_raw.ssid
    :field dest_ssid: ax25_frame.ax25_header.dest_ssid_raw.ssid
    :field ctl: ax25_frame.ax25_header.ctl
    :field pid: ax25_frame.payload.pid
    :field packet_type: ax25_frame.payload.payload.beacon.beacon_header.packet_type
    :field apid: ax25_frame.payload.payload.beacon.beacon_header.apid
    :field sequence_count: ax25_frame.payload.payload.beacon.beacon_header.sequence_count
    :field length: ax25_frame.payload.payload.beacon.beacon_header.length
    :field service_type: ax25_frame.payload.payload.beacon.beacon_header.service_type
    :field sub_service_type: ax25_frame.payload.payload.beacon.beacon_header.sub_service_type
    :field obc_boot_image: ax25_frame.payload.payload.beacon.obc_boot_image
    :field on_board_time: ax25_frame.payload.payload.beacon.on_board_time
    :field uptime: ax25_frame.payload.payload.beacon.uptime
    :field spacecraft_mode: ax25_frame.payload.payload.beacon.spacecraft_mode
    :field seperation_sequence_state: ax25_frame.payload.payload.beacon.seperation_sequence_state
    :field solar_array_deployment_state_0: ax25_frame.payload.payload.beacon.solar_array_deployment_state_0
    :field solar_array_deployment_state_1: ax25_frame.payload.payload.beacon.solar_array_deployment_state_1
    :field solar_array_deployment_state_2: ax25_frame.payload.payload.beacon.solar_array_deployment_state_2
    :field solar_array_deployment_state_3: ax25_frame.payload.payload.beacon.solar_array_deployment_state_3
    :field antenna_deployment_state_0: ax25_frame.payload.payload.beacon.antenna_deployment_state_0
    :field antenna_deployment_state_1: ax25_frame.payload.payload.beacon.antenna_deployment_state_1
    :field antenna_deployment_state_2: ax25_frame.payload.payload.beacon.antenna_deployment_state_2
    :field antenna_deployment_state_3: ax25_frame.payload.payload.beacon.antenna_deployment_state_3
    :field antenna_deployment_state_4: ax25_frame.payload.payload.beacon.antenna_deployment_state_4
    :field antenna_deployment_state_5: ax25_frame.payload.payload.beacon.antenna_deployment_state_5
    :field antenna_deployment_state_6: ax25_frame.payload.payload.beacon.antenna_deployment_state_6
    :field antenna_deployment_state_7: ax25_frame.payload.payload.beacon.antenna_deployment_state_7
    :field adm_soft_fire_counter: ax25_frame.payload.payload.beacon.adm_soft_fire_counter
    :field adm_hard_fire_counter: ax25_frame.payload.payload.beacon.adm_hard_fire_counter
    :field sadm_check_counter: ax25_frame.payload.payload.beacon.sadm_check_counter
    :field i2c_nack_addr_count: ax25_frame.payload.payload.beacon.i2c_nack_addr_count
    :field i2c_hw_state_error_count: ax25_frame.payload.payload.beacon.i2c_hw_state_error_count
    :field i2c_isr_error_count: ax25_frame.payload.payload.beacon.i2c_isr_error_count
    :field battery_current_direction: ax25_frame.payload.payload.beacon.battery_current_direction
    :field battery_current_0: ax25_frame.payload.payload.beacon.battery_current_0
    :field battery_current_1_msb: ax25_frame.payload.payload.beacon.battery_current_1_msb
    :field adm_telemetry_0: ax25_frame.payload.payload.beacon.adm_telemetry_0
    :field adm_telemetry_1: ax25_frame.payload.payload.beacon.adm_telemetry_1
    :field adm_telemetry_2: ax25_frame.payload.payload.beacon.adm_telemetry_2
    :field adm_telemetry_3: ax25_frame.payload.payload.beacon.adm_telemetry_3
    :field adm_telemetry_4: ax25_frame.payload.payload.beacon.adm_telemetry_4
    :field adm_telemetry_5: ax25_frame.payload.payload.beacon.adm_telemetry_5
    :field adm_telemetry_6: ax25_frame.payload.payload.beacon.adm_telemetry_6
    :field adm_telemetry_7: ax25_frame.payload.payload.beacon.adm_telemetry_7
    :field adm_telemetry_8: ax25_frame.payload.payload.beacon.adm_telemetry_8
    :field adm_telemetry_9: ax25_frame.payload.payload.beacon.adm_telemetry_9
    :field sadm_telemetry_0: ax25_frame.payload.payload.beacon.sadm_telemetry_0
    :field sadm_telemetry_1: ax25_frame.payload.payload.beacon.sadm_telemetry_1
    :field sadm_telemetry_2: ax25_frame.payload.payload.beacon.sadm_telemetry_2
    :field sadm_telemetry_3: ax25_frame.payload.payload.beacon.sadm_telemetry_3
    :field sadm_telemetry_4: ax25_frame.payload.payload.beacon.sadm_telemetry_4
    :field sadm_telemetry_5: ax25_frame.payload.payload.beacon.sadm_telemetry_5
    :field sadm_telemetry_6: ax25_frame.payload.payload.beacon.sadm_telemetry_6
    :field sadm_telemetry_7: ax25_frame.payload.payload.beacon.sadm_telemetry_7
    :field sadm_telemetry_8: ax25_frame.payload.payload.beacon.sadm_telemetry_8
    :field sadm_telemetry_9: ax25_frame.payload.payload.beacon.sadm_telemetry_9
    :field battery_current_1_lsbs: ax25_frame.payload.payload.beacon.battery_current_1_lsbs
    :field battery_current_2: ax25_frame.payload.payload.beacon.battery_current_2
    :field battery_voltage_0: ax25_frame.payload.payload.beacon.battery_voltage_0
    :field battery_voltage_1: ax25_frame.payload.payload.beacon.battery_voltage_1
    :field battery_voltage_2: ax25_frame.payload.payload.beacon.battery_voltage_2
    :field battery_temperature: ax25_frame.payload.payload.beacon.battery_temperature
    :field solar_array_current_0: ax25_frame.payload.payload.beacon.solar_array_current_0
    :field solar_array_voltage_0: ax25_frame.payload.payload.beacon.solar_array_voltage_0
    :field solar_array_voltage_1: ax25_frame.payload.payload.beacon.solar_array_voltage_1
    :field solar_array_voltage_2: ax25_frame.payload.payload.beacon.solar_array_voltage_2
    :field solar_array_voltage_3: ax25_frame.payload.payload.beacon.solar_array_voltage_3
    :field solar_array_voltage_4: ax25_frame.payload.payload.beacon.solar_array_voltage_4
    :field solar_array_voltage_5: ax25_frame.payload.payload.beacon.solar_array_voltage_5
    :field solar_array_voltage_6: ax25_frame.payload.payload.beacon.solar_array_voltage_6
    :field solar_array_voltage_7: ax25_frame.payload.payload.beacon.solar_array_voltage_7
    :field solar_array_voltage_8: ax25_frame.payload.payload.beacon.solar_array_voltage_8
    :field eps_bus_voltage_0: ax25_frame.payload.payload.beacon.eps_bus_voltage_0
    :field eps_bus_voltage_1: ax25_frame.payload.payload.beacon.eps_bus_voltage_1
    :field eps_bus_voltage_2: ax25_frame.payload.payload.beacon.eps_bus_voltage_2
    :field eps_bus_voltage_3: ax25_frame.payload.payload.beacon.eps_bus_voltage_3
    :field eps_bus_current_0: ax25_frame.payload.payload.beacon.eps_bus_current_0
    :field eps_bus_current_1: ax25_frame.payload.payload.beacon.eps_bus_current_1
    :field eps_bus_current_2: ax25_frame.payload.payload.beacon.eps_bus_current_2
    :field eps_bus_current_3: ax25_frame.payload.payload.beacon.eps_bus_current_3
    :field adcs_raw_gyro_rate_0: ax25_frame.payload.payload.beacon.adcs_raw_gyro_rate_0
    :field adcs_raw_gyro_rate_1: ax25_frame.payload.payload.beacon.adcs_raw_gyro_rate_1
    :field adcs_raw_gyro_rate_2: ax25_frame.payload.payload.beacon.adcs_raw_gyro_rate_2
    :field adcs_mtq_direction_duty_0: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_0
    :field adcs_mtq_direction_duty_1: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_1
    :field adcs_mtq_direction_duty_2: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_2
    :field adcs_mtq_direction_duty_3: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_3
    :field adcs_mtq_direction_duty_4: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_4
    :field adcs_mtq_direction_duty_5: ax25_frame.payload.payload.beacon.adcs_mtq_direction_duty_5
    :field adcs_status: ax25_frame.payload.payload.beacon.adcs_status
    :field adcs_bus_voltage_0: ax25_frame.payload.payload.beacon.adcs_bus_voltage_0
    :field adcs_bus_voltage_1: ax25_frame.payload.payload.beacon.adcs_bus_voltage_1
    :field adcs_bus_voltage_2: ax25_frame.payload.payload.beacon.adcs_bus_voltage_2
    :field adcs_bus_current_0: ax25_frame.payload.payload.beacon.adcs_bus_current_0
    :field adcs_bus_current_1: ax25_frame.payload.payload.beacon.adcs_bus_current_1
    :field adcs_bus_current_2: ax25_frame.payload.payload.beacon.adcs_bus_current_2
    :field adcs_board_temperature: ax25_frame.payload.payload.beacon.adcs_board_temperature
    :field adcs_adc_reference: ax25_frame.payload.payload.beacon.adcs_adc_reference
    :field adcs_sensor_current: ax25_frame.payload.payload.beacon.adcs_sensor_current
    :field adcs_mtq_current: ax25_frame.payload.payload.beacon.adcs_mtq_current
    :field adcs_array_temperature_0: ax25_frame.payload.payload.beacon.adcs_array_temperature_0
    :field adcs_array_temperature_1: ax25_frame.payload.payload.beacon.adcs_array_temperature_1
    :field adcs_array_temperature_2: ax25_frame.payload.payload.beacon.adcs_array_temperature_2
    :field adcs_array_temperature_3: ax25_frame.payload.payload.beacon.adcs_array_temperature_3
    :field adcs_array_temperature_4: ax25_frame.payload.payload.beacon.adcs_array_temperature_4
    :field adcs_array_temperature_5: ax25_frame.payload.payload.beacon.adcs_array_temperature_5
    :field adcs_css_raw_0: ax25_frame.payload.payload.beacon.adcs_css_raw_0
    :field adcs_css_raw_1: ax25_frame.payload.payload.beacon.adcs_css_raw_1
    :field adcs_css_raw_2: ax25_frame.payload.payload.beacon.adcs_css_raw_2
    :field adcs_css_raw_3: ax25_frame.payload.payload.beacon.adcs_css_raw_3
    :field adcs_css_raw_4: ax25_frame.payload.payload.beacon.adcs_css_raw_4
    :field adcs_css_raw_5: ax25_frame.payload.payload.beacon.adcs_css_raw_5
    :field fss_active_0: ax25_frame.payload.payload.beacon.fss_active_0
    :field fss_active_1: ax25_frame.payload.payload.beacon.fss_active_1
    :field fss_active_2: ax25_frame.payload.payload.beacon.fss_active_2
    :field fss_active_3: ax25_frame.payload.payload.beacon.fss_active_3
    :field fss_active_4: ax25_frame.payload.payload.beacon.fss_active_4
    :field fss_active_5: ax25_frame.payload.payload.beacon.fss_active_5
    :field css_active_selected_0: ax25_frame.payload.payload.beacon.css_active_selected_0
    :field css_active_selected_1: ax25_frame.payload.payload.beacon.css_active_selected_1
    :field css_active_selected_2: ax25_frame.payload.payload.beacon.css_active_selected_2
    :field css_active_selected_3: ax25_frame.payload.payload.beacon.css_active_selected_3
    :field css_active_selected_4: ax25_frame.payload.payload.beacon.css_active_selected_4
    :field css_active_selected_5: ax25_frame.payload.payload.beacon.css_active_selected_5
    :field adcs_sun_processed_0: ax25_frame.payload.payload.beacon.adcs_sun_processed_0
    :field adcs_sun_processed_1: ax25_frame.payload.payload.beacon.adcs_sun_processed_1
    :field adcs_sun_processed_2: ax25_frame.payload.payload.beacon.adcs_sun_processed_2
    :field adcs_detumble_counter: ax25_frame.payload.payload.beacon.adcs_detumble_counter
    :field adcs_mode: ax25_frame.payload.payload.beacon.adcs_mode
    :field adcs_state: ax25_frame.payload.payload.beacon.adcs_state
    :field cmc_rx_lock: ax25_frame.payload.payload.beacon.cmc_rx_lock
    :field cmc_rx_frame_count: ax25_frame.payload.payload.beacon.cmc_rx_frame_count
    :field cmc_rx_packet_count: ax25_frame.payload.payload.beacon.cmc_rx_packet_count
    :field cmc_rx_dropped_error_count: ax25_frame.payload.payload.beacon.cmc_rx_dropped_error_count
    :field cmc_rx_crc_error_count: ax25_frame.payload.payload.beacon.cmc_rx_crc_error_count
    :field cmc_rx_overrun_error_count: ax25_frame.payload.payload.beacon.cmc_rx_overrun_error_count
    :field cmc_rx_protocol_error_count: ax25_frame.payload.payload.beacon.cmc_rx_protocol_error_count
    :field cmc_smps_temperature: ax25_frame.payload.payload.beacon.cmc_smps_temperature
    :field cmc_pa_temperature: ax25_frame.payload.payload.beacon.cmc_pa_temperature
    :field ax25_mux_channel_enabled_0: ax25_frame.payload.payload.beacon.ax25_mux_channel_enabled_0
    :field ax25_mux_channel_enabled_1: ax25_frame.payload.payload.beacon.ax25_mux_channel_enabled_1
    :field ax25_mux_channel_enabled_2: ax25_frame.payload.payload.beacon.ax25_mux_channel_enabled_2
    :field digipeater_enabled: ax25_frame.payload.payload.beacon.digipeater_enabled
    :field pacsat_broadcast_enabled: ax25_frame.payload.payload.beacon.pacsat_broadcast_enabled
    :field pacsat_broadcast_in_progress: ax25_frame.payload.payload.beacon.pacsat_broadcast_in_progress
    :field paramvalid_flags_0: ax25_frame.payload.payload.beacon.paramvalid_flags_0
    :field paramvalid_flags_1: ax25_frame.payload.payload.beacon.paramvalid_flags_1
    :field paramvalid_flags_2: ax25_frame.payload.payload.beacon.paramvalid_flags_2
    :field paramvalid_flags_3: ax25_frame.payload.payload.beacon.paramvalid_flags_3
    :field paramvalid_flags_4: ax25_frame.payload.payload.beacon.paramvalid_flags_4
    :field paramvalid_flags_5: ax25_frame.payload.payload.beacon.paramvalid_flags_5
    :field paramvalid_flags_6: ax25_frame.payload.payload.beacon.paramvalid_flags_6
    :field paramvalid_flags_7: ax25_frame.payload.payload.beacon.paramvalid_flags_7
    :field paramvalid_flags_8: ax25_frame.payload.payload.beacon.paramvalid_flags_8
    :field paramvalid_flags_9: ax25_frame.payload.payload.beacon.paramvalid_flags_9
    :field paramvalid_flags_10: ax25_frame.payload.payload.beacon.paramvalid_flags_10
    :field paramvalid_flags_11: ax25_frame.payload.payload.beacon.paramvalid_flags_11
    :field paramvalid_flags_12: ax25_frame.payload.payload.beacon.paramvalid_flags_12
    :field paramvalid_flags_13: ax25_frame.payload.payload.beacon.paramvalid_flags_13
    :field paramvalid_flags_14: ax25_frame.payload.payload.beacon.paramvalid_flags_14
    :field paramvalid_flags_15: ax25_frame.payload.payload.beacon.paramvalid_flags_15
    :field paramvalid_flags_16: ax25_frame.payload.payload.beacon.paramvalid_flags_16
    :field paramvalid_flags_17: ax25_frame.payload.payload.beacon.paramvalid_flags_17
    :field paramvalid_flags_18: ax25_frame.payload.payload.beacon.paramvalid_flags_18
    :field paramvalid_flags_19: ax25_frame.payload.payload.beacon.paramvalid_flags_19
    :field paramvalid_flags_20: ax25_frame.payload.payload.beacon.paramvalid_flags_20
    :field paramvalid_flags_21: ax25_frame.payload.payload.beacon.paramvalid_flags_21
    :field paramvalid_flags_22: ax25_frame.payload.payload.beacon.paramvalid_flags_22
    :field paramvalid_flags_23: ax25_frame.payload.payload.beacon.paramvalid_flags_23
    :field paramvalid_flags_24: ax25_frame.payload.payload.beacon.paramvalid_flags_24
    :field paramvalid_flags_25: ax25_frame.payload.payload.beacon.paramvalid_flags_25
    :field paramvalid_flags_26: ax25_frame.payload.payload.beacon.paramvalid_flags_26
    :field paramvalid_flags_27: ax25_frame.payload.payload.beacon.paramvalid_flags_27
    :field paramvalid_flags_28: ax25_frame.payload.payload.beacon.paramvalid_flags_28
    :field paramvalid_flags_29: ax25_frame.payload.payload.beacon.paramvalid_flags_29
    :field paramvalid_flags_30: ax25_frame.payload.payload.beacon.paramvalid_flags_30
    :field paramvalid_flags_31: ax25_frame.payload.payload.beacon.paramvalid_flags_31
    :field paramvalid_flags_32: ax25_frame.payload.payload.beacon.paramvalid_flags_32
    :field paramvalid_flags_33: ax25_frame.payload.payload.beacon.paramvalid_flags_33
    :field paramvalid_flags_34: ax25_frame.payload.payload.beacon.paramvalid_flags_34
    :field paramvalid_flags_35: ax25_frame.payload.payload.beacon.paramvalid_flags_35
    :field paramvalid_flags_36: ax25_frame.payload.payload.beacon.paramvalid_flags_36
    :field paramvalid_flags_37: ax25_frame.payload.payload.beacon.paramvalid_flags_37
    :field paramvalid_flags_38: ax25_frame.payload.payload.beacon.paramvalid_flags_38
    :field paramvalid_flags_39: ax25_frame.payload.payload.beacon.paramvalid_flags_39
    :field checksumbytes: ax25_frame.payload.payload.beacon.checksumbytes
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.ax25_frame = Mirsat1.Ax25Frame(self._io, self, self._root)

    class Ax25Frame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ax25_header = Mirsat1.Ax25Header(self._io, self, self._root)
            _on = (self.ax25_header.ctl & 19)
            if _on == 0:
                self.payload = Mirsat1.IFrame(self._io, self, self._root)
            elif _on == 3:
                self.payload = Mirsat1.UiFrame(self._io, self, self._root)
            elif _on == 19:
                self.payload = Mirsat1.UiFrame(self._io, self, self._root)
            elif _on == 16:
                self.payload = Mirsat1.IFrame(self._io, self, self._root)
            elif _on == 18:
                self.payload = Mirsat1.IFrame(self._io, self, self._root)
            elif _on == 2:
                self.payload = Mirsat1.IFrame(self._io, self, self._root)


    class Ax25Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.dest_callsign_raw = Mirsat1.CallsignRaw(self._io, self, self._root)
            self.dest_ssid_raw = Mirsat1.SsidMask(self._io, self, self._root)
            self.src_callsign_raw = Mirsat1.CallsignRaw(self._io, self, self._root)
            self.src_ssid_raw = Mirsat1.SsidMask(self._io, self, self._root)
            self.ctl = self._io.read_u1()


    class UiFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.payload = Mirsat1.PayloadT(self._io, self, self._root)


    class Callsign(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.callsign = (self._io.read_bytes(6)).decode(u"ASCII")
            if not  ((self.callsign == u"3B8MRC") or (self.callsign == u"3B8MIR")) :
                raise kaitaistruct.ValidationNotAnyOfError(self.callsign, self._io, u"/types/callsign/seq/0")


    class BeaconAT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon_header = Mirsat1.BeaconHeaderT(self._io, self, self._root)
            self.start_byte = self._io.read_bytes(1)
            if not self.start_byte == b"\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x00", self.start_byte, self._io, u"/types/beacon_a_t/seq/1")
            self.obc_boot_image = self._io.read_bits_int_be(8)
            self.on_board_time = self._io.read_bits_int_be(32)
            self.uptime = self._io.read_bits_int_be(32)
            self.spacecraft_mode = self._io.read_bits_int_be(3)
            self.seperation_sequence_state = self._io.read_bits_int_be(4)
            self.solar_array_deployment_state_0 = self._io.read_bits_int_be(4)
            self.solar_array_deployment_state_1 = self._io.read_bits_int_be(4)
            self.solar_array_deployment_state_2 = self._io.read_bits_int_be(4)
            self.solar_array_deployment_state_3 = self._io.read_bits_int_be(4)
            self.antenna_deployment_state_0 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_1 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_2 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_3 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_4 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_5 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_6 = self._io.read_bits_int_be(16)
            self.antenna_deployment_state_7 = self._io.read_bits_int_be(16)
            self.adm_soft_fire_counter = self._io.read_bits_int_be(8)
            self.adm_hard_fire_counter = self._io.read_bits_int_be(8)
            self.adm_telemetry_0_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_0_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_0_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_1_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_1_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_1_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_2_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_2_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_2_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_3_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_3_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_3_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_4_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_4_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_4_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_5_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_5_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_5_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_6_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_6_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_6_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_7_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_7_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_7_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_8_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_8_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_8_l = self._io.read_bits_int_be(16)
            self.adm_telemetry_9_h = self._io.read_bits_int_be(32)
            self.adm_telemetry_9_m = self._io.read_bits_int_be(32)
            self.adm_telemetry_9_l = self._io.read_bits_int_be(16)
            self.sadm_check_counter = self._io.read_bits_int_be(5)
            self.sadm_telemetry_0_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_0_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_1_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_1_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_2_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_2_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_3_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_3_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_4_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_4_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_5_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_5_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_6_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_6_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_7_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_7_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_8_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_8_l = self._io.read_bits_int_be(32)
            self.sadm_telemetry_9_h = self._io.read_bits_int_be(32)
            self.sadm_telemetry_9_l = self._io.read_bits_int_be(32)
            self.i2c_nack_addr_count = self._io.read_bits_int_be(32)
            self.i2c_hw_state_error_count = self._io.read_bits_int_be(32)
            self.i2c_isr_error_count = self._io.read_bits_int_be(32)
            self.battery_current_direction = self._io.read_bits_int_be(1) != 0
            self.battery_current_0 = self._io.read_bits_int_be(10)
            self.battery_current_1_msb = self._io.read_bits_int_be(1) != 0

        @property
        def sadm_telemetry_4(self):
            if hasattr(self, '_m_sadm_telemetry_4'):
                return self._m_sadm_telemetry_4 if hasattr(self, '_m_sadm_telemetry_4') else None

            self._m_sadm_telemetry_4 = ((self.sadm_telemetry_4_h << 32) | self.sadm_telemetry_4_l)
            return self._m_sadm_telemetry_4 if hasattr(self, '_m_sadm_telemetry_4') else None

        @property
        def adm_telemetry_9(self):
            if hasattr(self, '_m_adm_telemetry_9'):
                return self._m_adm_telemetry_9 if hasattr(self, '_m_adm_telemetry_9') else None

            self._m_adm_telemetry_9 = (((self.adm_telemetry_9_h << 48) | (self.adm_telemetry_9_m << 32)) | self.adm_telemetry_9_l)
            return self._m_adm_telemetry_9 if hasattr(self, '_m_adm_telemetry_9') else None

        @property
        def adm_telemetry_7(self):
            if hasattr(self, '_m_adm_telemetry_7'):
                return self._m_adm_telemetry_7 if hasattr(self, '_m_adm_telemetry_7') else None

            self._m_adm_telemetry_7 = (((self.adm_telemetry_7_h << 48) | (self.adm_telemetry_7_m << 32)) | self.adm_telemetry_7_l)
            return self._m_adm_telemetry_7 if hasattr(self, '_m_adm_telemetry_7') else None

        @property
        def adm_telemetry_4(self):
            if hasattr(self, '_m_adm_telemetry_4'):
                return self._m_adm_telemetry_4 if hasattr(self, '_m_adm_telemetry_4') else None

            self._m_adm_telemetry_4 = (((self.adm_telemetry_4_h << 48) | (self.adm_telemetry_4_m << 32)) | self.adm_telemetry_4_l)
            return self._m_adm_telemetry_4 if hasattr(self, '_m_adm_telemetry_4') else None

        @property
        def adm_telemetry_6(self):
            if hasattr(self, '_m_adm_telemetry_6'):
                return self._m_adm_telemetry_6 if hasattr(self, '_m_adm_telemetry_6') else None

            self._m_adm_telemetry_6 = (((self.adm_telemetry_6_h << 48) | (self.adm_telemetry_6_m << 32)) | self.adm_telemetry_6_l)
            return self._m_adm_telemetry_6 if hasattr(self, '_m_adm_telemetry_6') else None

        @property
        def sadm_telemetry_1(self):
            if hasattr(self, '_m_sadm_telemetry_1'):
                return self._m_sadm_telemetry_1 if hasattr(self, '_m_sadm_telemetry_1') else None

            self._m_sadm_telemetry_1 = ((self.sadm_telemetry_1_h << 32) | self.sadm_telemetry_1_l)
            return self._m_sadm_telemetry_1 if hasattr(self, '_m_sadm_telemetry_1') else None

        @property
        def sadm_telemetry_6(self):
            if hasattr(self, '_m_sadm_telemetry_6'):
                return self._m_sadm_telemetry_6 if hasattr(self, '_m_sadm_telemetry_6') else None

            self._m_sadm_telemetry_6 = ((self.sadm_telemetry_6_h << 32) | self.sadm_telemetry_6_l)
            return self._m_sadm_telemetry_6 if hasattr(self, '_m_sadm_telemetry_6') else None

        @property
        def adm_telemetry_3(self):
            if hasattr(self, '_m_adm_telemetry_3'):
                return self._m_adm_telemetry_3 if hasattr(self, '_m_adm_telemetry_3') else None

            self._m_adm_telemetry_3 = (((self.adm_telemetry_3_h << 48) | (self.adm_telemetry_3_m << 16)) | self.adm_telemetry_3_l)
            return self._m_adm_telemetry_3 if hasattr(self, '_m_adm_telemetry_3') else None

        @property
        def sadm_telemetry_0(self):
            if hasattr(self, '_m_sadm_telemetry_0'):
                return self._m_sadm_telemetry_0 if hasattr(self, '_m_sadm_telemetry_0') else None

            self._m_sadm_telemetry_0 = ((self.sadm_telemetry_0_h << 32) | self.sadm_telemetry_0_l)
            return self._m_sadm_telemetry_0 if hasattr(self, '_m_sadm_telemetry_0') else None

        @property
        def adm_telemetry_8(self):
            if hasattr(self, '_m_adm_telemetry_8'):
                return self._m_adm_telemetry_8 if hasattr(self, '_m_adm_telemetry_8') else None

            self._m_adm_telemetry_8 = (((self.adm_telemetry_8_h << 48) | (self.adm_telemetry_8_m << 32)) | self.adm_telemetry_8_l)
            return self._m_adm_telemetry_8 if hasattr(self, '_m_adm_telemetry_8') else None

        @property
        def adm_telemetry_0(self):
            if hasattr(self, '_m_adm_telemetry_0'):
                return self._m_adm_telemetry_0 if hasattr(self, '_m_adm_telemetry_0') else None

            self._m_adm_telemetry_0 = (((self.adm_telemetry_0_h << 48) | (self.adm_telemetry_0_m << 16)) | self.adm_telemetry_0_l)
            return self._m_adm_telemetry_0 if hasattr(self, '_m_adm_telemetry_0') else None

        @property
        def sadm_telemetry_3(self):
            if hasattr(self, '_m_sadm_telemetry_3'):
                return self._m_sadm_telemetry_3 if hasattr(self, '_m_sadm_telemetry_3') else None

            self._m_sadm_telemetry_3 = ((self.sadm_telemetry_3_h << 32) | self.sadm_telemetry_3_l)
            return self._m_sadm_telemetry_3 if hasattr(self, '_m_sadm_telemetry_3') else None

        @property
        def sadm_telemetry_9(self):
            if hasattr(self, '_m_sadm_telemetry_9'):
                return self._m_sadm_telemetry_9 if hasattr(self, '_m_sadm_telemetry_9') else None

            self._m_sadm_telemetry_9 = ((self.sadm_telemetry_9_h << 32) | self.sadm_telemetry_9_l)
            return self._m_sadm_telemetry_9 if hasattr(self, '_m_sadm_telemetry_9') else None

        @property
        def adm_telemetry_5(self):
            if hasattr(self, '_m_adm_telemetry_5'):
                return self._m_adm_telemetry_5 if hasattr(self, '_m_adm_telemetry_5') else None

            self._m_adm_telemetry_5 = (((self.adm_telemetry_5_h << 48) | (self.adm_telemetry_5_m << 32)) | self.adm_telemetry_5_l)
            return self._m_adm_telemetry_5 if hasattr(self, '_m_adm_telemetry_5') else None

        @property
        def sadm_telemetry_5(self):
            if hasattr(self, '_m_sadm_telemetry_5'):
                return self._m_sadm_telemetry_5 if hasattr(self, '_m_sadm_telemetry_5') else None

            self._m_sadm_telemetry_5 = ((self.sadm_telemetry_5_h << 32) | self.sadm_telemetry_5_l)
            return self._m_sadm_telemetry_5 if hasattr(self, '_m_sadm_telemetry_5') else None

        @property
        def adm_telemetry_2(self):
            if hasattr(self, '_m_adm_telemetry_2'):
                return self._m_adm_telemetry_2 if hasattr(self, '_m_adm_telemetry_2') else None

            self._m_adm_telemetry_2 = (((self.adm_telemetry_2_h << 48) | (self.adm_telemetry_2_m << 16)) | self.adm_telemetry_2_l)
            return self._m_adm_telemetry_2 if hasattr(self, '_m_adm_telemetry_2') else None

        @property
        def sadm_telemetry_2(self):
            if hasattr(self, '_m_sadm_telemetry_2'):
                return self._m_sadm_telemetry_2 if hasattr(self, '_m_sadm_telemetry_2') else None

            self._m_sadm_telemetry_2 = ((self.sadm_telemetry_2_h << 32) | self.sadm_telemetry_2_l)
            return self._m_sadm_telemetry_2 if hasattr(self, '_m_sadm_telemetry_2') else None

        @property
        def sadm_telemetry_8(self):
            if hasattr(self, '_m_sadm_telemetry_8'):
                return self._m_sadm_telemetry_8 if hasattr(self, '_m_sadm_telemetry_8') else None

            self._m_sadm_telemetry_8 = ((self.sadm_telemetry_8_h << 32) | self.sadm_telemetry_8_l)
            return self._m_sadm_telemetry_8 if hasattr(self, '_m_sadm_telemetry_8') else None

        @property
        def sadm_telemetry_7(self):
            if hasattr(self, '_m_sadm_telemetry_7'):
                return self._m_sadm_telemetry_7 if hasattr(self, '_m_sadm_telemetry_7') else None

            self._m_sadm_telemetry_7 = ((self.sadm_telemetry_7_h << 32) | self.sadm_telemetry_7_l)
            return self._m_sadm_telemetry_7 if hasattr(self, '_m_sadm_telemetry_7') else None

        @property
        def adm_telemetry_1(self):
            if hasattr(self, '_m_adm_telemetry_1'):
                return self._m_adm_telemetry_1 if hasattr(self, '_m_adm_telemetry_1') else None

            self._m_adm_telemetry_1 = (((self.adm_telemetry_1_h << 48) | (self.adm_telemetry_1_m << 16)) | self.adm_telemetry_1_l)
            return self._m_adm_telemetry_1 if hasattr(self, '_m_adm_telemetry_1') else None


    class BeaconHeaderT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.packet_type = self._io.read_u1()
            self.apid = self._io.read_u1()
            self.sequence_count = self._io.read_u2be()
            self.length = self._io.read_u2be()
            self.reserved = self._io.read_u1()
            self.service_type = self._io.read_u1()
            self.sub_service_type = self._io.read_u1()


    class BeaconT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.beacon = self._io.read_bytes_full()


    class IFrame(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.pid = self._io.read_u1()
            self.ax25_info = self._io.read_bytes_full()


    class SsidMask(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.ssid_mask = self._io.read_u1()

        @property
        def ssid(self):
            if hasattr(self, '_m_ssid'):
                return self._m_ssid if hasattr(self, '_m_ssid') else None

            self._m_ssid = ((self.ssid_mask & 15) >> 1)
            return self._m_ssid if hasattr(self, '_m_ssid') else None


    class PayloadT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.header = self._io.read_bytes(6)
            _on = self._root.frametype
            if _on == 793:
                self.beacon = Mirsat1.BeaconAT(self._io, self, self._root)
            else:
                self.beacon = Mirsat1.BeaconBT(self._io, self, self._root)


    class CallsignRaw(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self._raw__raw_callsign_ror = self._io.read_bytes(6)
            self._raw_callsign_ror = KaitaiStream.process_rotate_left(self._raw__raw_callsign_ror, 8 - (1), 1)
            _io__raw_callsign_ror = KaitaiStream(BytesIO(self._raw_callsign_ror))
            self.callsign_ror = Mirsat1.Callsign(_io__raw_callsign_ror, self, self._root)


    class BeaconBT(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.battery_current_1_lsbs = self._io.read_bits_int_be(9)
            self.battery_current_2 = self._io.read_bits_int_be(10)
            self.battery_voltage_0 = self._io.read_bits_int_be(10)
            self.battery_voltage_1 = self._io.read_bits_int_be(10)
            self.battery_voltage_2 = self._io.read_bits_int_be(10)
            self.battery_temperature = self._io.read_bits_int_be(10)
            self.solar_array_current_0 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_0 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_1 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_2 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_3 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_4 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_5 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_6 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_7 = self._io.read_bits_int_be(10)
            self.solar_array_voltage_8 = self._io.read_bits_int_be(10)
            self.eps_bus_voltage_0 = self._io.read_bits_int_be(10)
            self.eps_bus_voltage_1 = self._io.read_bits_int_be(10)
            self.eps_bus_voltage_2 = self._io.read_bits_int_be(10)
            self.eps_bus_voltage_3 = self._io.read_bits_int_be(10)
            self.eps_bus_current_0 = self._io.read_bits_int_be(10)
            self.eps_bus_current_1 = self._io.read_bits_int_be(10)
            self.eps_bus_current_2 = self._io.read_bits_int_be(10)
            self.eps_bus_current_3 = self._io.read_bits_int_be(10)
            self.adcs_raw_gyro_rate_0_bits = self._io.read_bits_int_be(16)
            self.adcs_raw_gyro_rate_1_bits = self._io.read_bits_int_be(16)
            self.adcs_raw_gyro_rate_2_bits = self._io.read_bits_int_be(16)
            self.adcs_mtq_direction_duty_0 = self._io.read_bits_int_be(8)
            self.adcs_mtq_direction_duty_1 = self._io.read_bits_int_be(8)
            self.adcs_mtq_direction_duty_2 = self._io.read_bits_int_be(8)
            self.adcs_mtq_direction_duty_3 = self._io.read_bits_int_be(8)
            self.adcs_mtq_direction_duty_4 = self._io.read_bits_int_be(8)
            self.adcs_mtq_direction_duty_5 = self._io.read_bits_int_be(8)
            self.adcs_status = self._io.read_bits_int_be(16)
            self.adcs_bus_voltage_0 = self._io.read_bits_int_be(16)
            self.adcs_bus_voltage_1 = self._io.read_bits_int_be(16)
            self.adcs_bus_voltage_2 = self._io.read_bits_int_be(16)
            self.adcs_bus_current_0 = self._io.read_bits_int_be(16)
            self.adcs_bus_current_1 = self._io.read_bits_int_be(16)
            self.adcs_bus_current_2 = self._io.read_bits_int_be(16)
            self.adcs_board_temperature = self._io.read_bits_int_be(16)
            self.adcs_adc_reference = self._io.read_bits_int_be(16)
            self.adcs_sensor_current = self._io.read_bits_int_be(16)
            self.adcs_mtq_current = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_0 = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_1 = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_2 = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_3 = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_4 = self._io.read_bits_int_be(16)
            self.adcs_array_temperature_5 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_0 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_1 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_2 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_3 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_4 = self._io.read_bits_int_be(16)
            self.adcs_css_raw_5 = self._io.read_bits_int_be(16)
            self.fss_active_0 = self._io.read_bits_int_be(2)
            self.fss_active_1 = self._io.read_bits_int_be(2)
            self.fss_active_2 = self._io.read_bits_int_be(2)
            self.fss_active_3 = self._io.read_bits_int_be(2)
            self.fss_active_4 = self._io.read_bits_int_be(2)
            self.fss_active_5 = self._io.read_bits_int_be(2)
            self.css_active_selected_0 = self._io.read_bits_int_be(2)
            self.css_active_selected_1 = self._io.read_bits_int_be(2)
            self.css_active_selected_2 = self._io.read_bits_int_be(2)
            self.css_active_selected_3 = self._io.read_bits_int_be(2)
            self.css_active_selected_4 = self._io.read_bits_int_be(2)
            self.css_active_selected_5 = self._io.read_bits_int_be(2)
            self.adcs_sun_processed_0 = self._io.read_bits_int_be(16)
            self.adcs_sun_processed_1 = self._io.read_bits_int_be(16)
            self.adcs_sun_processed_2 = self._io.read_bits_int_be(16)
            self.reserved_0 = self._io.read_bits_int_be(16)
            self.reserved_1 = self._io.read_bits_int_be(16)
            self.reserved_2 = self._io.read_bits_int_be(16)
            self.reserved_3 = self._io.read_bits_int_be(16)
            self.adcs_detumble_counter = self._io.read_bits_int_be(16)
            self.adcs_mode = self._io.read_bits_int_be(16)
            self.adcs_state = self._io.read_bits_int_be(16)
            self.reserved_4 = self._io.read_bits_int_be(10)
            self.reserved_5 = self._io.read_bits_int_be(4)
            self.reserved_6 = self._io.read_bits_int_be(16)
            self.reserved_7 = self._io.read_bits_int_be(3)
            self.reserved_8 = self._io.read_bits_int_be(4)
            self.cmc_rx_lock = self._io.read_bits_int_be(1) != 0
            self.cmc_rx_frame_count = self._io.read_bits_int_be(16)
            self.cmc_rx_packet_count = self._io.read_bits_int_be(16)
            self.cmc_rx_dropped_error_count = self._io.read_bits_int_be(16)
            self.cmc_rx_crc_error_count = self._io.read_bits_int_be(16)
            self.cmc_rx_overrun_error_count = self._io.read_bits_int_be(8)
            self.cmc_rx_protocol_error_count = self._io.read_bits_int_be(16)
            self.cmc_smps_temperature_bits = self._io.read_bits_int_be(8)
            self.cmc_pa_temperature_bits = self._io.read_bits_int_be(8)
            self.ax25_mux_channel_enabled_0 = self._io.read_bits_int_be(1) != 0
            self.ax25_mux_channel_enabled_1 = self._io.read_bits_int_be(1) != 0
            self.ax25_mux_channel_enabled_2 = self._io.read_bits_int_be(1) != 0
            self.digipeater_enabled = self._io.read_bits_int_be(1) != 0
            self.pacsat_broadcast_enabled = self._io.read_bits_int_be(1) != 0
            self.pacsat_broadcast_in_progress = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_0 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_1 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_2 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_3 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_4 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_5 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_6 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_7 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_8 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_9 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_10 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_11 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_12 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_13 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_14 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_15 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_16 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_17 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_18 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_19 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_20 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_21 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_22 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_23 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_24 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_25 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_26 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_27 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_28 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_29 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_30 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_31 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_32 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_33 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_34 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_35 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_36 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_37 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_38 = self._io.read_bits_int_be(1) != 0
            self.paramvalid_flags_39 = self._io.read_bits_int_be(1) != 0
            self.padding = self._io.read_bits_int_be(5)
            self.checksumbytes = self._io.read_bits_int_be(16)
            self._io.align_to_byte()
            self.tail = self._io.read_bytes(97)

        @property
        def cmc_pa_temperature(self):
            if hasattr(self, '_m_cmc_pa_temperature'):
                return self._m_cmc_pa_temperature if hasattr(self, '_m_cmc_pa_temperature') else None

            self._m_cmc_pa_temperature = ((-1 * (self.cmc_pa_temperature_bits & 127)) if (self.cmc_pa_temperature_bits & 128) == 1 else (self.cmc_pa_temperature_bits & 127))
            return self._m_cmc_pa_temperature if hasattr(self, '_m_cmc_pa_temperature') else None

        @property
        def adcs_raw_gyro_rate_1(self):
            if hasattr(self, '_m_adcs_raw_gyro_rate_1'):
                return self._m_adcs_raw_gyro_rate_1 if hasattr(self, '_m_adcs_raw_gyro_rate_1') else None

            self._m_adcs_raw_gyro_rate_1 = ((-1 * (self.adcs_raw_gyro_rate_1_bits & 32767)) if (self.adcs_raw_gyro_rate_1_bits & 32768) == 1 else (self.adcs_raw_gyro_rate_1_bits & 32767))
            return self._m_adcs_raw_gyro_rate_1 if hasattr(self, '_m_adcs_raw_gyro_rate_1') else None

        @property
        def adcs_raw_gyro_rate_2(self):
            if hasattr(self, '_m_adcs_raw_gyro_rate_2'):
                return self._m_adcs_raw_gyro_rate_2 if hasattr(self, '_m_adcs_raw_gyro_rate_2') else None

            self._m_adcs_raw_gyro_rate_2 = ((-1 * (self.adcs_raw_gyro_rate_1_bits & 32767)) if (self.adcs_raw_gyro_rate_1_bits & 32768) == 1 else (self.adcs_raw_gyro_rate_1_bits & 32767))
            return self._m_adcs_raw_gyro_rate_2 if hasattr(self, '_m_adcs_raw_gyro_rate_2') else None

        @property
        def adcs_raw_gyro_rate_0(self):
            if hasattr(self, '_m_adcs_raw_gyro_rate_0'):
                return self._m_adcs_raw_gyro_rate_0 if hasattr(self, '_m_adcs_raw_gyro_rate_0') else None

            self._m_adcs_raw_gyro_rate_0 = ((-1 * (self.adcs_raw_gyro_rate_0_bits & 32767)) if (self.adcs_raw_gyro_rate_0_bits & 32768) == 1 else (self.adcs_raw_gyro_rate_0_bits & 32767))
            return self._m_adcs_raw_gyro_rate_0 if hasattr(self, '_m_adcs_raw_gyro_rate_0') else None

        @property
        def cmc_smps_temperature(self):
            if hasattr(self, '_m_cmc_smps_temperature'):
                return self._m_cmc_smps_temperature if hasattr(self, '_m_cmc_smps_temperature') else None

            self._m_cmc_smps_temperature = ((-1 * (self.cmc_smps_temperature_bits & 127)) if (self.cmc_smps_temperature_bits & 128) == 1 else (self.cmc_smps_temperature_bits & 127))
            return self._m_cmc_smps_temperature if hasattr(self, '_m_cmc_smps_temperature') else None


    @property
    def framelength(self):
        if hasattr(self, '_m_framelength'):
            return self._m_framelength if hasattr(self, '_m_framelength') else None

        self._m_framelength = self._io.size()
        return self._m_framelength if hasattr(self, '_m_framelength') else None

    @property
    def frametype(self):
        if hasattr(self, '_m_frametype'):
            return self._m_frametype if hasattr(self, '_m_frametype') else None

        _pos = self._io.pos()
        self._io.seek(29)
        self._m_frametype = self._io.read_u2be()
        self._io.seek(_pos)
        return self._m_frametype if hasattr(self, '_m_frametype') else None


